package com.sharix.sportsmanfriend;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;

import java.util.Objects;


public abstract class BaseDialogClass extends AppCompatDialogFragment {

    protected BaseDialogClass.DialogListener listener;
    protected View view;
    private Dialog dialog;

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(requireActivity());

        dialog = builder.setView(view).create();

        WindowManager.LayoutParams lp = new WindowManager.LayoutParams();
        lp.copyFrom(dialog.getWindow().getAttributes());
        lp.width = WindowManager.LayoutParams.MATCH_PARENT;
        lp.height = WindowManager.LayoutParams.WRAP_CONTENT;
        dialog.getWindow().setAttributes(lp);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        return dialog;
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        try {
            listener = (BaseDialogClass.DialogListener) getTargetFragment();
        }catch (ClassCastException e) {
            throw new ClassCastException(Objects.requireNonNull(getTargetFragment()) + "must implement Example Dialog listener");
        }

    }

    public interface DialogListener{
        void applyInfo(Bundle bundle);
    }

    protected void addClickEventToAllViews(){
        Button btn_apply = view.findViewById(R.id.btn_apply);
        Button btn_cancel = view.findViewById(R.id.btn_cancel);

        btn_apply.setOnClickListener(v -> {
            listener.applyInfo(DataToTransfer());
            dialog.cancel();
        });

        btn_cancel.setOnClickListener(v -> dialog.cancel());
    }

    protected abstract Bundle DataToTransfer();
}