package com.sharix.sportsmanfriend;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;

import org.osmdroid.api.IGeoPoint;
import org.osmdroid.api.IMapController;
import org.osmdroid.tileprovider.tilesource.TileSourceFactory;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;

import java.text.SimpleDateFormat;
import java.util.Locale;

public class MapHelperClass {

    public static void configureMap(Context context, MapView inMap, double latitude, double longitude, int zoom) {
        inMap.setTileSource(TileSourceFactory.DEFAULT_TILE_SOURCE);
        IsDarkMap(context, inMap);
        inMap.setMultiTouchControls(true);
        inMap.setMaxZoomLevel(18.0);

        IMapController mapController = inMap.getController();
        mapController.setZoom(zoom);
        GeoPoint startPoint = new GeoPoint(latitude, longitude);
        mapController.setCenter(startPoint);

        inMap.setBuiltInZoomControls(false);
    }

    public static void zoomIn(MapView inMap){
        if (inMap.canZoomIn()) {
            inMap.getController().zoomIn();
        }
    }

    public static void zoomOut(MapView inMap){
        if (inMap.canZoomOut()) {
            inMap.getController().zoomOut();
        }
    }

    public static void setCenterOfMap(MapView inMap, double latitude, double longitude){
        inMap.getController().setCenter(new GeoPoint(latitude, longitude));
    }

    public static void setZoomOfMap(MapView inMap, int zoom){
        inMap.getController().setZoom(zoom);
    }

    public static IGeoPoint getMapCenter(MapView inMap){
        return inMap.getMapCenter();
    }

    public static int getMapZoom(MapView inMap){
        return inMap.getZoomLevel();
    }

    private static void IsDarkMap(Context context, MapView inMap){
        int nightModeFlags =
                context.getResources().getConfiguration().uiMode &
                        Configuration.UI_MODE_NIGHT_MASK;
        if (nightModeFlags == Configuration.UI_MODE_NIGHT_YES) {
            SimpleDateFormat format = new SimpleDateFormat("HH", Locale.getDefault());
            ColorMatrix inverseMatrix = new ColorMatrix(new float[]{
                    -1.0f, 0.0f, 0.0f, 0.0f, 255f,
                    0.0f, -1.0f, 0.0f, 0.0f, 255f,
                    0.0f, 0.0f, -1.0f, 0.0f, 255f,
                    0.0f, 0.0f, 0.0f, 1.0f, 0.0f
            });

            int destinationColor = Color.parseColor("#FF2A2A2A");
            float lr = (255.0f - Color.red(destinationColor)) / 255.0f;
            float lg = (255.0f - Color.green(destinationColor)) / 255.0f;
            float lb = (255.0f - Color.blue(destinationColor)) / 255.0f;
            ColorMatrix grayscaleMatrix = new ColorMatrix(new float[]{
                    lr, lg, lb, 0, 0, //
                    lr, lg, lb, 0, 0, //
                    lr, lg, lb, 0, 0, //
                    0, 0, 0, 0, 255, //
            });
            grayscaleMatrix.preConcat(inverseMatrix);
            int dr = Color.red(destinationColor);
            int dg = Color.green(destinationColor);
            int db = Color.blue(destinationColor);
            float drf = dr / 255f;
            float dgf = dg / 255f;
            float dbf = db / 255f;
            ColorMatrix tintMatrix = new ColorMatrix(new float[]{
                    drf, 0, 0, 0, 0, //
                    0, dgf, 0, 0, 0, //
                    0, 0, dbf, 0, 0, //
                    0, 0, 0, 1, 0, //
            });
            tintMatrix.preConcat(grayscaleMatrix);
            float lDestination = drf * lr + dgf * lg + dbf * lb;
            float scale = 1f - lDestination;
            float translate = 1 - scale * 0.5f;
            ColorMatrix scaleMatrix = new ColorMatrix(new float[]{
                    scale, 0, 0, 0, dr * translate, //
                    0, scale, 0, 0, dg * translate, //
                    0, 0, scale, 0, db * translate, //
                    0, 0, 0, 1, 0, //
            });
            scaleMatrix.preConcat(tintMatrix);
            ColorMatrixColorFilter filter = new ColorMatrixColorFilter(scaleMatrix);
            inMap.getOverlayManager().getTilesOverlay().setColorFilter(filter);
        }
    }
}
