package com.sharix.sportsmanfriend.dialogs;

import android.Manifest;
import android.app.Dialog;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.widget.ImageButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.sharix.sportsmanfriend.BaseDialogClass;
import com.sharix.sportsmanfriend.MapHelperClass;
import com.sharix.sportsmanfriend.R;

import org.osmdroid.views.MapView;

import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.Locale;

public class SelectLocationDialog extends BaseDialogClass {
    private final int PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 2;
    private ImageButton btnZoomIn, btnZoomOut, btnMyLocation;
    private MapView map = null;
    //определяет последнюю известную локацию
    private FusedLocationProviderClient fusedLocationProviderClient;

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = requireActivity().getLayoutInflater();
        view = inflater.inflate(R.layout.dialog_select_location, null);

        map = view.findViewById(R.id.dialog_map);
        btnZoomIn = view.findViewById(R.id.btn_zoomIn);
        btnZoomOut = view.findViewById(R.id.btn_zoomOut);
        btnMyLocation = view.findViewById(R.id.btnMyLocation);

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireActivity());

        MapHelperClass.configureMap(requireActivity() ,map,55.7522200, 37.6155600, 15);

        addClickEventToAllViews();

        int nightModeFlags =
                getContext().getResources().getConfiguration().uiMode &
                        Configuration.UI_MODE_NIGHT_MASK;
        if (nightModeFlags == Configuration.UI_MODE_NIGHT_YES) {
            SimpleDateFormat format = new SimpleDateFormat("HH", Locale.getDefault());
            //taking cue from https://medium.com/square-corner-blog/welcome-to-the-color-matrix-64d112e3f43d
            ColorMatrix inverseMatrix = new ColorMatrix(new float[] {
                    -1.0f, 0.0f, 0.0f, 0.0f, 255f,
                    0.0f, -1.0f, 0.0f, 0.0f, 255f,
                    0.0f, 0.0f, -1.0f, 0.0f, 255f,
                    0.0f, 0.0f, 0.0f, 1.0f, 0.0f
            });

            int destinationColor = Color.parseColor("#FF2A2A2A");
            float lr = (255.0f - Color.red(destinationColor))/255.0f;
            float lg = (255.0f - Color.green(destinationColor))/255.0f;
            float lb = (255.0f - Color.blue(destinationColor))/255.0f;
            ColorMatrix grayscaleMatrix = new ColorMatrix(new float[] {
                    lr, lg, lb, 0, 0, //
                    lr, lg, lb, 0, 0, //
                    lr, lg, lb, 0, 0, //
                    0, 0, 0, 0, 255, //
            });
            grayscaleMatrix.preConcat(inverseMatrix);
            int dr = Color.red(destinationColor);
            int dg = Color.green(destinationColor);
            int db = Color.blue(destinationColor);
            float drf = dr / 255f;
            float dgf = dg / 255f;
            float dbf = db / 255f;
            ColorMatrix tintMatrix = new ColorMatrix(new float[] {
                    drf, 0, 0, 0, 0, //
                    0, dgf, 0, 0, 0, //
                    0, 0, dbf, 0, 0, //
                    0, 0, 0, 1, 0, //
            });
            tintMatrix.preConcat(grayscaleMatrix);
            float lDestination = drf * lr + dgf * lg + dbf * lb;
            float scale = 1f - lDestination;
            float translate = 1 - scale * 0.5f;
            ColorMatrix scaleMatrix = new ColorMatrix(new float[] {
                    scale, 0, 0, 0, dr * translate, //
                    0, scale, 0, 0, dg * translate, //
                    0, 0, scale, 0, db * translate, //
                    0, 0, 0, 1, 0, //
            });
            scaleMatrix.preConcat(tintMatrix);
            ColorMatrixColorFilter filter = new ColorMatrixColorFilter(scaleMatrix);
            map.getOverlayManager().getTilesOverlay().setColorFilter(filter);
        }

        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected Bundle DataToTransfer() {
        Bundle bundle = new Bundle();
        bundle.putString("FromBlock", "LocationBlock");
        bundle.putDouble("latitude", MapHelperClass.getMapCenter(map).getLatitude());
        bundle.putDouble("longitude", MapHelperClass.getMapCenter(map).getLongitude());
        bundle.putByteArray("screenshot", getScreenshot());
        return bundle;
    }

    @Override
    protected void addClickEventToAllViews() {
        super.addClickEventToAllViews();

        btnZoomIn.setOnClickListener(v -> MapHelperClass.zoomIn(map));

        btnZoomOut.setOnClickListener(v -> MapHelperClass.zoomOut(map));

        btnMyLocation.setOnClickListener(v -> {
            if (ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                    && ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permission = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};
                requestPermissions(permission, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
            }

            fusedLocationProviderClient.getLastLocation()
                .addOnSuccessListener(requireActivity(), location -> {
                    if (location != null) {
                        MapHelperClass.setCenterOfMap(map, location.getLatitude(), location.getLongitude());
                        MapHelperClass.setZoomOfMap(map, 18);
                    }
                });

        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults); {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                requireActivity().recreate();
            }
        }
    }

    private byte[] getScreenshot(){

        float density = getResources().getDisplayMetrics().density;
        int pixels = (int) (100 * density);

        Bitmap screenshot = Bitmap.createBitmap(pixels, pixels, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(screenshot);
        int centerX = view.getWidth() / 2;
        int centerY = view.getHeight() / 2;
        canvas.translate(-centerX + pixels / 2, -centerY + pixels / 2);
        view.draw(canvas);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        screenshot.compress(Bitmap.CompressFormat.PNG, 100, baos);
        return baos.toByteArray();
    }
}
