package com.sharix.sportsmanfriend.dialogs;

import android.app.Dialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.widget.TimePicker;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.sharix.sportsmanfriend.BaseDialogClass;
import com.sharix.sportsmanfriend.R;

public class TimeOfEventDialog extends BaseDialogClass {

    private TimePicker timePickerStart, timePickerEnd;

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = requireActivity().getLayoutInflater();
        view = inflater.inflate(R.layout.dialog_time_of_event, null);

        timePickerStart = view.findViewById(R.id.timePickerStart);
        timePickerStart.setIs24HourView(true);
        timePickerEnd = view.findViewById(R.id.timePickerEnd);
        timePickerEnd.setIs24HourView(true);

        addClickEventToAllViews();

        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected Bundle DataToTransfer() {
        Bundle bundle = new Bundle();
        bundle.putString("FromBlock", "TimeBlock");
        bundle.putInt("HourStart", timePickerStart.getHour());
        bundle.putInt("MinuteStart", timePickerStart.getMinute());
        bundle.putInt("HourEnd", timePickerEnd.getHour());
        bundle.putInt("MinuteEnd", timePickerEnd.getMinute());
        return bundle;
    }

    @Override
    protected void addClickEventToAllViews() {
        super.addClickEventToAllViews();

        timePickerStart.setOnTimeChangedListener((view, hourOfDay, minute) -> {
            if (timePickerEnd.getHour() < hourOfDay || (timePickerEnd.getHour() == hourOfDay && timePickerEnd.getMinute() <= minute)) {
                timePickerEnd.setHour(hourOfDay);
                timePickerEnd.setMinute(Math.max(minute + 1, timePickerEnd.getMinute()));
            }
        });

        timePickerEnd.setOnTimeChangedListener((view, hourOfDay, minute) -> {
            if (timePickerStart.getHour() > hourOfDay || (timePickerStart.getHour() == hourOfDay && timePickerStart.getMinute() >= minute)) {
                timePickerStart.setHour(hourOfDay);
                timePickerStart.setMinute(Math.min(minute - 1, timePickerStart.getMinute()));
            }
        });
    }
}