package com.sharix.sportsmanfriend.view;

import android.os.Bundle;
import android.preference.PreferenceManager;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import com.sharix.sportsmanfriend.BaseDialogClass;
import com.sharix.sportsmanfriend.BuildConfig;
import com.sharix.sportsmanfriend.OnBackPressedListener;
import com.sharix.sportsmanfriend.R;
import com.sharix.sportsmanfriend.databinding.ActivityMainBinding;
import com.sharix.sportsmanfriend.view.fragments.CalendarFragment;
import com.sharix.sportsmanfriend.view.fragments.ChatsFragment;
import com.sharix.sportsmanfriend.view.fragments.CreateEventFragment;
import com.sharix.sportsmanfriend.view.fragments.MapFragment;
import com.sharix.sportsmanfriend.view.fragments.MoreFragment;

import org.osmdroid.config.Configuration;

public class MainActivity extends AppCompatActivity implements OnBackPressedListener, BaseDialogClass.DialogListener {

    ActivityMainBinding binding;

    MapFragment mapFragment = new MapFragment();
    CalendarFragment calendarFragment = new CalendarFragment();
    CreateEventFragment createEventFragment = new CreateEventFragment();
    ChatsFragment chatsFragment = new ChatsFragment();
    MoreFragment moreFragment = new MoreFragment();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // установка user agent
        Configuration.getInstance().load(getApplicationContext(),
                PreferenceManager.getDefaultSharedPreferences(getApplicationContext()));

        Configuration.getInstance().setUserAgentValue(BuildConfig.APPLICATION_ID);

        binding = ActivityMainBinding.inflate(getLayoutInflater());

        setContentView(binding.getRoot());

        replaceFragment(mapFragment);

        binding.bottomNavigationView.setOnItemSelectedListener(item -> {
            switch (item.getItemId()){

                case R.id.home:
                    replaceFragment(mapFragment);
                    break;

                case R.id.today:
                    replaceFragment(calendarFragment);
                    break;

                case R.id.history:
                    replaceFragment(createEventFragment);
                    break;

                case R.id.chats:
                    replaceFragment(chatsFragment);
                    break;

                case R.id.more:
                    replaceFragment(moreFragment);
                    break;
            }

            return true;
        });
    }

    private void replaceFragment(Fragment fragment){
        FragmentManager fragmentManager = getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        fragmentTransaction.replace(R.id.frameWindow,fragment);
        //fragmentTransaction.addToBackStack(null);
        fragmentTransaction.commit();
    }

    @Override
    public void onBackPressed() {
        // Получаем текущий фрагмент
        Fragment currentFragment = getSupportFragmentManager().findFragmentById(R.id.frameWindow);

        // Если текущий фрагмент реализует интерфейс OnBackPressedListener, вызываем его метод onBackPressed()
        if (currentFragment instanceof OnBackPressedListener) {
            ((OnBackPressedListener) currentFragment).onBackPressed();
        } else {
            super.onBackPressed();
        }
    }

    @Override
    public void applyInfo(Bundle bundle) {

    }
}