package com.sharix.sportsmanfriend.view.fragments;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.sharix.sportsmanfriend.BaseFragmentClass;
import com.sharix.sportsmanfriend.MapHelperClass;
import com.sharix.sportsmanfriend.R;

import org.osmdroid.views.MapView;

public class MapFragment extends BaseFragmentClass {
    // константы
    private final int REQUEST_PERMISSIONS_REQUEST_CODE = 1, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 2;
    private final String KEY_ZOOM_LEVEL = "zoomLevel", KEY_CENTER_LATITUDE = "centerLatitude", KEY_CENTER_LONGITUDE = "centerLongitude";
    //Элементы
    private MapView map = null;
    private ImageButton btnZoomIn, btnZoomOut, btnMyLocation, btnFilters;
    private LinearLayout btnOpenEventList;
    //определяет последнюю известную локацию
    private FusedLocationProviderClient fusedLocationProviderClient;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_map, container, false);

        btnZoomIn = view.findViewById(R.id.btn_zoomIn);
        btnZoomOut = view.findViewById(R.id.btn_zoomOut);
        btnMyLocation = view.findViewById(R.id.btnMyLocation);
        btnOpenEventList = view.findViewById(R.id.btnOpenEventList);
        btnFilters = view.findViewById(R.id.btnFilters);

        map = view.findViewById(R.id.map);
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireActivity());

        if (!hasSavedState()){
            MapHelperClass.configureMap(requireActivity(), map,55.934473, 37.497081, 15);
        }

        askPermissions();
        addClickEventToAllViews();

        return view;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_PERMISSIONS_REQUEST_CODE: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    requireActivity().recreate();
                }
            }
            case PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    requireActivity().recreate();
                }
            }
        }
    }

    private void askPermissions() {
        if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.INTERNET) != PackageManager.PERMISSION_GRANTED ||
            ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED ||
            ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            String[] permission = {Manifest.permission.INTERNET, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.ACCESS_FINE_LOCATION};
            requestPermissions(permission, REQUEST_PERMISSIONS_REQUEST_CODE);
        }
    }

    private void addClickEventToAllViews() {
        btnZoomIn.setOnClickListener(v -> MapHelperClass.zoomIn(map));

        btnZoomOut.setOnClickListener(v -> MapHelperClass.zoomOut(map));

        btnMyLocation.setOnClickListener(v -> {
            if (ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permission = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};
                requestPermissions(permission, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
            }

            fusedLocationProviderClient.getLastLocation()
                    .addOnSuccessListener(requireActivity(), location -> {
                        if (location != null) {
                            MapHelperClass.setCenterOfMap(map, location.getLatitude(), location.getLongitude());
                            MapHelperClass.setZoomOfMap(map, 18);
                        }
                    });

        });

        btnOpenEventList.setOnClickListener(v -> {
            FragmentManager fragmentManager = requireActivity().getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            EventListFragment myFragment = new EventListFragment();
            fragmentTransaction.replace(R.id.frameWindow, myFragment);
            fragmentTransaction.addToBackStack(null);
            fragmentTransaction.commit();
        });

        btnFilters.setOnClickListener(v -> {
            FragmentManager fragmentManager = requireActivity().getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            EventFiltersFragment myFragment = new EventFiltersFragment();
            fragmentTransaction.replace(R.id.frameWindow, myFragment);
            fragmentTransaction.addToBackStack(null);
            fragmentTransaction.commit();
        });
    }

    @Override
    protected boolean hasSavedState() {
        Bundle state = getSavedState();

        if (state == null) {
            return false;
        }

        MapHelperClass.configureMap(requireActivity(), map, state.getDouble(KEY_CENTER_LATITUDE), state.getDouble(KEY_CENTER_LONGITUDE), state.getInt(KEY_ZOOM_LEVEL));

        return true;
    }

    @Override
    protected Bundle getStateToSave() {
        Bundle state = new Bundle();

        state.putInt(KEY_ZOOM_LEVEL, MapHelperClass.getMapZoom(map));
        state.putDouble(KEY_CENTER_LATITUDE, MapHelperClass.getMapCenter(map).getLatitude());
        state.putDouble(KEY_CENTER_LONGITUDE, MapHelperClass.getMapCenter(map).getLongitude());

        return state;
    }
}